"use strict";
// Copyright (c) Frédéric Maquin <fred@ephread.com>
// Licensed under the MIT License.
// See LICENSE in the project root for license information.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const vscode_uri_1 = require("vscode-uri");
const compile_1 = require("./compile");
const types_1 = require("./types");
const utils_1 = require("./utils");
/* Properties */
/******************************************************************************/
/** Capabilities defined by the client. Defaults to none. */
const capabilities = {
    configuration: false,
    workspaceFolder: false,
    diagnostic: false
};
/** Connection with the client. */
const connection = vscode_languageserver_1.createConnection(vscode_languageserver_1.ProposedFeatures.all);
const logger = new types_1.InkConnectionLogger(connection);
/** Documents managed by the clients. */
const documents = new vscode_languageserver_1.TextDocuments();
/**
 * Workspace directories managed by the server,
 * mapped from the workspace managed by the client.
 *
 * URIs of the client's `WorkspaceFolder` will be used as keys.
 */
const workspaceDirectories = new Map();
/**
 * Document settings, scoped per document, returned by the client.
 *
 * URIs of the client's `TextDocument` will be used as keys.
 */
const documentSettings = new Map();
/* Helpers */
/******************************************************************************/
/**
 * Returns the InkWorkspace of the given document. If no workspace could be found,
 * returns `undefined`.
 *
 * Since Inklecate requires as a full compilation of the entire related project
 * to push diagnostics, the workspace of each document must be known.
 *
 * @param document a text document, belonging to the returned workspace.
 */
function getInkWorkspaceOfDocument(document) {
    const documentPath = vscode_uri_1.default.parse(document.uri).fsPath;
    for (const workspaceKey of workspaceDirectories.keys()) {
        const workspace = workspaceDirectories.get(workspaceKey);
        if (!workspace) {
            continue;
        }
        const workspacePath = vscode_uri_1.default.parse(workspace.folder.uri).fsPath;
        if (utils_1.isFilePathChildOfDirPath(documentPath, workspacePath)) {
            return workspace;
        }
    }
    return undefined;
}
/**
 * Fetch the configuration settings of the given document, from the client.
 *
 * @param document the document of which fetch the configuration settings.
 */
function fetchDocumentConfigurationSettings(document) {
    if (!capabilities.configuration) {
        return Promise.resolve(Object.assign({}, compile_1.DEFAULT_SETTINGS));
    }
    let result = documentSettings.get(document.uri);
    if (!result) {
        result = connection.workspace.getConfiguration({
            scopeUri: document.uri,
            section: "ink"
        });
        documentSettings.set(document.uri, result);
    }
    return result;
}
/**
 * Send the given errors to the client.
 *
 * @param errors the errors to push.
 */
function pushDiagnostics(errors) {
    for (const textDocument of documents.all()) {
        const diagnostics = [];
        for (const error of errors) {
            if (vscode_uri_1.default.parse(textDocument.uri).fsPath === error.filePath) {
                const diagnostic = {
                    severity: utils_1.getDiagnosticSeverityFromInkErrorType(error.type),
                    range: {
                        start: { line: error.lineNumber - 1, character: 0 },
                        end: { line: error.lineNumber, character: 0 }
                    },
                    message: error.message,
                    source: "inklecate"
                };
                diagnostics.push(diagnostic);
            }
        }
        connection.sendDiagnostics({ uri: textDocument.uri, diagnostics });
    }
}
/**
 * Initialize workspaces by fetching opened `WorkspaceFolder` from the client and
 * creating the temporary directories which will hold copies of the Ink Project.
 */
function initializeInkWorkspaces() {
    return __awaiter(this, void 0, void 0, function* () {
        const workspaceFolders = yield connection.workspace.getWorkspaceFolders();
        if (workspaceFolders) {
            for (const workspaceFolder of workspaceFolders) {
                let workspace = workspaceDirectories.get(workspaceFolder.uri);
                if (!workspace) {
                    workspace = {
                        folder: workspaceFolder
                    };
                    workspaceDirectories.set(workspaceFolder.uri, workspace);
                }
                compile_1.prepareTempDirectoryForCompilation(workspaceFolder, logger, tempDir => {
                    if (tempDir) {
                        workspace.temporaryCompilationDirectory = tempDir;
                    }
                    else {
                        reportServerError();
                    }
                });
            }
        }
    });
}
/**
 * Update the content of the given document, in the temporary copy of the workspace,
 * and compile the project.
 *
 * @param document the document to update.
 */
function updateDocumentAndCompileWorkspace(document) {
    return __awaiter(this, void 0, void 0, function* () {
        const workspace = getInkWorkspaceOfDocument(document);
        if (!workspace) {
            return;
        }
        const settings = yield fetchDocumentConfigurationSettings(document);
        compile_1.updateFile(document, workspace, error => {
            if (error) {
                connection.console.log(`Could not update '${document.uri}', ${error.message}`);
                reportServerError();
            }
            else {
                compile_1.compileProject(settings, workspace, logger, pushDiagnostics);
            }
        });
    });
}
function reportServerError() {
    connection.window.showErrorMessage("An unrecoverable error occured with Ink Language Server. " +
        "Please see the logs for more information.");
}
/* Connection callbacks */
/******************************************************************************/
connection.onInitialize((params) => {
    connection.console.log("Server Initialized");
    const clientCapabilities = params.capabilities;
    if (clientCapabilities.workspace) {
        capabilities.configuration = !!clientCapabilities.workspace.configuration;
        capabilities.workspaceFolder = !!clientCapabilities.workspace.workspaceFolders;
        if (clientCapabilities.textDocument) {
            capabilities.diagnostic = !!clientCapabilities.textDocument.publishDiagnostics;
        }
    }
    return {
        capabilities: {
            textDocumentSync: documents.syncKind
        }
    };
});
connection.onInitialized(() => {
    if (capabilities.configuration) {
        connection.client.register(vscode_languageserver_1.DidChangeConfigurationNotification.type, undefined);
    }
    if (capabilities.workspaceFolder) {
        connection.workspace.onDidChangeWorkspaceFolders(event => {
            initializeInkWorkspaces();
        });
    }
    initializeInkWorkspaces();
});
connection.onDidChangeConfiguration(change => {
    documentSettings.clear();
});
/* Document callbacks */
/******************************************************************************/
documents.onDidChangeContent(change => {
    updateDocumentAndCompileWorkspace(change.document);
});
documents.onDidClose(event => {
    documentSettings.delete(event.document.uri);
});
/* Final setup */
/******************************************************************************/
documents.listen(connection);
connection.listen();
//# sourceMappingURL=server.js.map